var path = require('path')
var md5 = require('md5')
var webpack = require('webpack')
var getWebpackConfig = require('./config')
var mfs = require('./mfs')
var packageJson = require('../package.json')

function setupCompiler (fileSystem, webpackConfig) {
  var compiler = webpack(webpackConfig)
  compiler.inputFileSystem = fileSystem
  compiler.resolvers.normal.fileSystem = fileSystem
  compiler.resolvers.context.fileSystem = fileSystem
  compiler.outputFileSystem = fileSystem
  return compiler
}

function compile (code) {
	return new Promise (function (resolve, reject) {
    var directory = path.join(process.cwd(), md5(code + JSON.stringify(packageJson)))
    var entryPoint = path.join(directory, 'entry.js')
    var outputPoint = path.join(directory, "output.js")

    // using cache
    if (mfs.existsSync(outputPoint)) {
      return resolve({
        logs: ['using cache'],
        code: mfs.readFileSync(outputPoint).toString()
      })
    }

    mfs.mkdirpSync(directory)
    mfs.writeFileSync(path.join(directory, 'App.vue'), code)
    mfs.writeFileSync(entryPoint, "\n      import App from './App.vue'\n      App.el = '#root'\n      new Vue(App)\n    ")

    var webpackConfig = getWebpackConfig(directory, entryPoint)
		var compiler = setupCompiler(mfs, webpackConfig)

		try {
			compiler.run(function (err, stats) {
        mfs.unlink(entryPoint, function () {})
				if (err) {
					return rejecter(err)
				}
        var logs = stats.toJson().errors
        var code = mfs.readFileSync(outputPoint).toString()
        // mfs.unlink(outputPoint, () => {})
        resolve({ logs: logs, code: code })
			})
		} catch (e) {
      mfs.unlink(entryPoint, function () {})
			reject(e)
		}
	})
}

module.exports = compile
