var fs = require('fs')
var path = require('path')
var recursive = require('recursive-readdir')

var loadedDeps = []
var nodeModulePath = path.resolve(__dirname, '../node_modules')

function preLoadLoader (mfs, name) {
  if (name === 'fsevents') { return }
  if (loadedDeps.indexOf(name) >= 0) { return }

  loadedDeps.push(name)

  setTimeout(function () {
    var loaderPath = path.resolve(nodeModulePath, name)
    var loaderConfig = fs.readFileSync(path.join(loaderPath, 'package.json'))
    var loaderConfigJson = JSON.parse(loaderConfig.toString())

    recursive(loaderPath, function (err, files) {
      files.forEach(function (file) {
        var relativeFile = file
        var paths = relativeFile.split(path.sep).slice(1)

        paths.reduce(function (currentPath, pathPart, index) {
          currentPath += path.sep + pathPart
          if (index < paths.length - 1 && !mfs.existsSync(currentPath)) {
            mfs.mkdirpSync(currentPath)
          }
          return currentPath
        }, '')

        if (path.extname(relativeFile)) {
          var content = fs.readFileSync(file, 'utf8')
          mfs.writeFileSync(relativeFile, content || ' ')
        }
      })
    })

    Object.keys(loaderConfigJson.dependencies || {}).forEach(function (dep) {
      preLoadLoader(mfs, dep)
    })
  }, 0)
}

module.exports = function (mfs, loaders) {
  loaders.forEach(function (loader) {
    preLoadLoader(mfs, loader)
  })
}
