const fs = require('fs')
const path = require('path')
const WeexRuntime = require('weex-js-runtime')
const Vue = require('weex-vue-framework')
const compile = require('../index')

function toFunction (content) {
  return `function () { ${content} }`
}

function readFile (filePath) {
  return fs.readFileSync(path.join(__dirname, filePath), 'utf8')
}

function isObject (object) {
  return object !== null && typeof object === 'object'
}

function omitUseless (object) {
  if (isObject(object)) {
    delete object.ref
    for (const key in object) {
      if (object[key] == '') {
        delete object[key]
      }
      omitUseless(object[key])
    }
    for (const key in object) {
      if (isObject(object[key]) && Object.keys(object[key]).length < 1) {
        delete object[key]
      }
      omitUseless(object[key])
    }
  }
  return object
}

function compileAndExecute (template, additional = '') {
  WeexRuntime.config.frameworks = { Vue }
  const context = WeexRuntime.init(WeexRuntime.config)
  context.registerModules({
    timer: ['setTimeout', 'setInterval']
  })
  return compile(template).then(({ code }) => {
    return new Promise(resolve => {
      const id = String(Date.now() * Math.random())
      const instance = context.createInstance(id, code)
      setTimeout(() => {
        resolve(omitUseless(instance.document.body.toJSON()))
        context.destroyInstance(id)
      }, 10)
    })
  })
}

module.exports = {
  compile,
  compileAndExecute
}
